#include "jDockApp/jDockApp.h"
#include "jDockApp/colors.h"
#include <getopt.h>
#include "jenglish.h"
#include <stdio.h>
#include <mysql/mysql.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define MAXLINE 256


struct conf {
   char *host;
   char *user;
   char *password;
   int frequency;
   char *file;
   int pad;
   int port;
   int gran;
} conf;

struct stuff {
   int first;
   int last_ques;
   int last_time;
   int last_sques;
   float max_rate;
};

struct stuff stuff = {1,0,0,0,0.0};
 
int dontread = 0;

void show_help() {
    printf("-s <sec>     :  Seconds between refresh, overriden by config file.\n");
    printf("-c <filename>:  Config file. Default ~/.wmjsqlrc.\n");
    exit(0);
}

void setup(int argc, char** argv) {
   char c;
   int  i=0, res;
   while( -1 != (c = getopt(argc, argv, "hrsc:"))) {
       switch(c) {
           case '?': exit(1);
           case 'h': show_help();

           case 's': i=atoi(optarg); break;
	   case 'c': conf.file=optarg; break;
       }
   }
   res = read_conf();
   if (i>0) {
      set_update_delay(i);
   } else if (conf.frequency) {
      set_update_delay(conf.frequency);
   } else {
      set_update_delay(30);
   }
   set_loop_delay(1000); /* mu seconds */
}

int read_conf(void) {
   FILE *fp;
   char buff[MAXLINE];
   char *tmp, *s1;

   if (!conf.file) {
      conf.file = getenv("HOME");
      strcat(conf.file, "/.wmjsql");
   }
   if (!(fp=fopen(conf.file, "r"))) {
      fprintf(stderr, "Error: Couldn't open config file: %s!\n", conf.file);
      exit(1);
   }
   while(fgets(buff,MAXLINE-1,fp)) {
      tmp = buff;
      while(*tmp && *tmp <= ' ')
         tmp++;
      switch(*tmp) {
         case 'h':
            if (s1 = strtok(tmp, ":")) {
               s1 = strtok(NULL, "\n");
               conf.host = strdup(s1);
            }
            break;
         case 'u':
            if (s1 = strtok(tmp, ":")) {
               s1 = strtok(NULL, "\n");
               conf.user = strdup(s1);
            }
            break;
         case 'p':
            if (s1 = strtok(tmp, ":")) {
	       if (strcmp(s1, "password") == 0) {
                  s1 = strtok(NULL, "\n");
                  conf.password = strdup(s1);
               } else if (strcmp(s1, "port")==0) {
	          s1 = strtok(NULL, "\n");
		  conf.port = atoi(s1);
               }
            }
            break;
	 case 'g':
	    if (s1 = strtok(tmp, ":")) {
	       s1 = strtok(NULL, "\n");
	       if (strcmp(s1, "minute")==0)
	          conf.gran = 1;
	    }
	    break;
         case 'f':
            if (s1 = strtok(tmp, ":")) {
               s1 = strtok(NULL, "\n");
               conf.frequency = atoi(s1);
            }
            break;
      }
   }
   fclose(fp);
   conf.host = (conf.host) ? conf.host : "localhost";
   conf.port = (conf.port) ? conf.port : 3306;
   if (strlen(conf.host) > 11) {
      fprintf(stderr,"Hmm, hostname too long to fit. Need to fix this...\n");
      exit(1);
   } else if (strlen(conf.host) > 8) {
      conf.pad = 1;
   } else {
      conf.pad = (11 - (strlen(conf.host)))/2;
   }
   conf.gran = (conf.gran) ? conf.gran : 0;
}

void do_update() {
   MYSQL mysql;
   MYSQL_RES *result;
   char *stat, *s1, *up, *threads, *questions, *otables, *slow;
   int delta_ques, delta_time, delta_slow, tmp, tmp1, tmp2;
   int i, res, fail=0;
   float rate;
   mysql_init(&mysql);
   if (!mysql_real_connect(&mysql,conf.host,conf.user,conf.password,"mysql",conf.port,NULL,0)) {
      fail=1;
   }
   if (fail) {
      clear_window();
      jpprintf(conf.pad, 0, YELLOW, "%s", conf.host);
      /*Print Failure*/
      jpprintf(2,2,RED,"Failure");
   } else {
      stat = strdup(mysql_stat(&mysql));
      mysql_close(&mysql);

      if (s1 = strtok(stat, " ")) {
         i = 1;
         while (s1 = strtok(NULL, " ")) {
            switch (i) {
               case 1 :
                  up = s1;
                  break;
               case 3 :
                  threads = s1;
                  break;
               case 5 :
                  questions = s1;
                  break;
	       case 7 :
	          slow = s1;
		  break;
               case 16 :
                  otables = s1;
                  break;
            }
            i++;
         }
      }
      tmp = atoi(questions);
      tmp1 = atoi(up);
      tmp2 = atoi(slow);
      if (stuff.first) {
         stuff.first = 0;
	 delta_ques = 0;
	 delta_time = 0;
      } else {
         delta_ques = tmp - stuff.last_ques;
	 delta_time = tmp1 - stuff.last_time;
	 delta_slow = tmp2 - stuff.last_sques;
      }
      stuff.last_ques = tmp;
      stuff.last_time = tmp1;
      stuff.last_sques = tmp2;
      clear_window();
      jpprintf(conf.pad, 0, YELLOW, "%s", conf.host);
      jpprintf(1, 1, BLUE, "Thrds ");
      jpprintf(6, 1, GREEN, ": ");
      jpprintf(7, 1, CYAN, "%d", atoi(threads));
      jpprintf(1, 2, BLUE, "OT");
      jpprintf(3, 2, GREEN, ":");
      jpprintf(6, 2, CYAN, "%s", otables);
      if (delta_time != 0) {
	 jpprintf(1,3,BLUE, "SQ");
	 jpprintf(3,3,GREEN,":");
	 if (delta_slow > 0) {
	    jpprintf(6,3,RED, "%d", delta_slow);
         } else {
	    jpprintf(6,3,CYAN, "%d", delta_slow);
	 }
	 if (conf.gran == 0) { /*queries per second*/
            rate = (float)delta_ques/(float)delta_time;
	    jpprintf(1,4,BLUE, "QPS");
	    jpprintf(4,4,GREEN, ":");
	    jpprintf(5,4,CYAN, "%.2f", rate);
	 } else { /*queries per minute*/
	    rate = (float)delta_ques/((float)delta_time/60);
	    jpprintf(1,4,BLUE, "QPM");
	    jpprintf(4,4,GREEN, ":");
	    jpprintf(6,4,CYAN, "%.0f", rate);
         }
	 stuff.max_rate = (stuff.max_rate < rate) ? rate : stuff.max_rate;
	 jpprintf(1,5,BLUE, "Max");
	 jpprintf(4,5,GREEN, ":");
	 if (conf.gran == 0) {
	    jpprintf(5,5,CYAN, "%.2f", stuff.max_rate);
         } else {
	    jpprintf(6,5,CYAN, "%.0f", stuff.max_rate);
         }
      }
   free(stat);
   }
}

void do_expose() {
    do_update();
}
    
void do_button_release() {
    do_update();
}
